#region Using declarations
using System;
using System.ComponentModel;
using System.Diagnostics;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.Xml.Serialization;
using NinjaTrader.Cbi;
using NinjaTrader.Data;
using NinjaTrader.Gui.Chart;
#endregion

// This namespace holds all indicators and is required. Do not change it.
namespace NinjaTrader.Indicator
{
    /// <summary>
    /// 
    /// </summary>
    [Description("Indicator as described in Sylvain Vervoort's 'Average True Range Trailing Stops' June 2009 S&C article.")]
    public class ATRTrailingStop : Indicator
    {
        #region Variables
		private int 	period	= 5;
		private double	multi	= 3.5;
        #endregion

        /// <summary>
        /// This method is used to configure the indicator and is called once before any bar data is loaded.
        /// </summary>
        protected override void Initialize()
        {
            Add(new Plot(Color.Blue, PlotStyle.Line, "Trailing Stop"));
            CalculateOnBarClose	= true;
            Overlay				= true;
			BarsRequired = 1;
        }

        /// <summary>
        /// Called on each bar update event (incoming tick)
        /// </summary>
        protected override void OnBarUpdate()
        {
			if (CurrentBar < 1)
				return;

			// Trailing stop
			double trail;
			double loss = ATR(Input, Period)[0] * Multi;
			
			if (Close[0] > Value[1] && Close[1] > Value[1])
				trail = Math.Max(Value[1], Close[0] - loss);
			
			else if (Close[0] < Value[1] && Close[1] < Value[1])
				trail = Math.Min(Value[1], Close[0] + loss);
				
			else if (Close[0] > Value[1])
			{
				trail = Close[0] - loss;
				DrawArrowDown(CurrentBar.ToString(), false, 1, Value[1], Color.Orange);
			}
			
			else
			{
				trail = Close[0] + loss;
				DrawArrowUp(CurrentBar.ToString(), false, 1, Value[1], Color.Orange);
			}

            Value.Set(trail);
        }
		
        #region Properties
        [Description("ATR period")]
        [Category("Parameters")]
        public int Period
        {
            get { return period; }
            set { period = Math.Max(1, value); }
        }
		
		[Description("ATR multiplication")]
        [Category("Parameters")]
        public double Multi
        {
            get { return multi; }
            set { multi = Math.Max(0, value); }
        }
        #endregion
    }
}

#region NinjaScript generated code. Neither change nor remove.
// This namespace holds all indicators and is required. Do not change it.
namespace NinjaTrader.Indicator
{
    public partial class Indicator : IndicatorBase
    {
        private ATRTrailingStop[] cacheATRTrailingStop = null;

        private static ATRTrailingStop checkATRTrailingStop = new ATRTrailingStop();

        /// <summary>
        /// Indicator as described in Sylvain Vervoort's 'Average True Range Trailing Stops' June 2009 S&C article.
        /// </summary>
        /// <returns></returns>
        public ATRTrailingStop ATRTrailingStop(double multi, int period)
        {
            return ATRTrailingStop(Input, multi, period);
        }

        /// <summary>
        /// Indicator as described in Sylvain Vervoort's 'Average True Range Trailing Stops' June 2009 S&C article.
        /// </summary>
        /// <returns></returns>
        public ATRTrailingStop ATRTrailingStop(Data.IDataSeries input, double multi, int period)
        {
            checkATRTrailingStop.Multi = multi;
            multi = checkATRTrailingStop.Multi;
            checkATRTrailingStop.Period = period;
            period = checkATRTrailingStop.Period;

            if (cacheATRTrailingStop != null)
                for (int idx = 0; idx < cacheATRTrailingStop.Length; idx++)
                    if (Math.Abs(cacheATRTrailingStop[idx].Multi - multi) <= double.Epsilon && cacheATRTrailingStop[idx].Period == period && cacheATRTrailingStop[idx].EqualsInput(input))
                        return cacheATRTrailingStop[idx];

            ATRTrailingStop indicator = new ATRTrailingStop();
            indicator.BarsRequired = BarsRequired;
            indicator.CalculateOnBarClose = CalculateOnBarClose;
            indicator.Input = input;
            indicator.Multi = multi;
            indicator.Period = period;
            indicator.SetUp();

            ATRTrailingStop[] tmp = new ATRTrailingStop[cacheATRTrailingStop == null ? 1 : cacheATRTrailingStop.Length + 1];
            if (cacheATRTrailingStop != null)
                cacheATRTrailingStop.CopyTo(tmp, 0);
            tmp[tmp.Length - 1] = indicator;
            cacheATRTrailingStop = tmp;
            Indicators.Add(indicator);

            return indicator;
        }

    }
}

// This namespace holds all market analyzer column definitions and is required. Do not change it.
namespace NinjaTrader.MarketAnalyzer
{
    public partial class Column : ColumnBase
    {
        /// <summary>
        /// Indicator as described in Sylvain Vervoort's 'Average True Range Trailing Stops' June 2009 S&C article.
        /// </summary>
        /// <returns></returns>
        [Gui.Design.WizardCondition("Indicator")]
        public Indicator.ATRTrailingStop ATRTrailingStop(double multi, int period)
        {
            return _indicator.ATRTrailingStop(Input, multi, period);
        }

        /// <summary>
        /// Indicator as described in Sylvain Vervoort's 'Average True Range Trailing Stops' June 2009 S&C article.
        /// </summary>
        /// <returns></returns>
        public Indicator.ATRTrailingStop ATRTrailingStop(Data.IDataSeries input, double multi, int period)
        {
            return _indicator.ATRTrailingStop(input, multi, period);
        }

    }
}

// This namespace holds all strategies and is required. Do not change it.
namespace NinjaTrader.Strategy
{
    public partial class Strategy : StrategyBase
    {
        /// <summary>
        /// Indicator as described in Sylvain Vervoort's 'Average True Range Trailing Stops' June 2009 S&C article.
        /// </summary>
        /// <returns></returns>
        [Gui.Design.WizardCondition("Indicator")]
        public Indicator.ATRTrailingStop ATRTrailingStop(double multi, int period)
        {
            return _indicator.ATRTrailingStop(Input, multi, period);
        }

        /// <summary>
        /// Indicator as described in Sylvain Vervoort's 'Average True Range Trailing Stops' June 2009 S&C article.
        /// </summary>
        /// <returns></returns>
        public Indicator.ATRTrailingStop ATRTrailingStop(Data.IDataSeries input, double multi, int period)
        {
            if (InInitialize && input == null)
                throw new ArgumentException("You only can access an indicator with the default input/bar series from within the 'Initialize()' method");

            return _indicator.ATRTrailingStop(input, multi, period);
        }

    }
}
#endregion
